@@ -20,8 +20,10 @@
  *
  */
 
-
+#ifdef HAVE_CONFIG_H
 #include <config.h>
+#endif
+
 #include <stdio.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -29,11 +31,18 @@
 #include <unistd.h>
 #include <string.h>
 #include <ctype.h>
+
 #include "gmime-part.h"
 #include "gmime-utils.h"
+#include "gmime-stream-mem.h"
+#include "gmime-stream-filter.h"
+#include "gmime-filter-basic.h"
 #include "md5-utils.h"
 
 
+#define NEEDS_DECODING(encoding)  (((GMimePartEncodingType) encoding) == GMIME_PART_ENCODING_BASE64 ||        \
+				   ((GMimePartEncodingType) encoding) == GMIME_PART_ENCODING_QUOTEDPRINTABLE)
+
 /**
  * g_mime_part_new: Create a new MIME Part object
  *
@@ -63,7 +72,7 @@ g_mime_part_new ()
  * Returns an empty MIME Part object with the specified content-type.
  **/
 GMimePart *
-g_mime_part_new_with_type (const gchar *type, const gchar *subtype)
+g_mime_part_new_with_type (const char *type, const char *subtype)
 {
 	GMimePart *mime_part;
 	
@@ -133,7 +142,7 @@ g_mime_part_destroy (GMimePart *mime_part)
 	}
 	
 	if (mime_part->content)
-		g_byte_array_free (mime_part->content, TRUE);
+		g_mime_data_wrapper_destroy (mime_part->content);
 	
 	g_free (mime_part);
 }
@@ -147,7 +156,7 @@ g_mime_part_destroy (GMimePart *mime_part)
  * Set the content description for the specified mime part.
  **/
 void
-g_mime_part_set_content_description (GMimePart *mime_part, const gchar *description)
+g_mime_part_set_content_description (GMimePart *mime_part, const char *description)
 {
 	g_return_if_fail (mime_part != NULL);
 	
@@ -164,7 +173,7 @@ g_mime_part_set_content_description (GMimePart *mime_part, const gchar *descript
  *
  * Returns the content description for the specified mime part.
  **/
-const gchar *
+const char *
 g_mime_part_get_content_description (const GMimePart *mime_part)
 {
 	g_return_val_if_fail (mime_part != NULL, NULL);
@@ -181,7 +190,7 @@ g_mime_part_get_content_description (const GMimePart *mime_part)
  * Set the content id for the specified mime part.
  **/
 void
-g_mime_part_set_content_id (GMimePart *mime_part, const gchar *content_id)
+g_mime_part_set_content_id (GMimePart *mime_part, const char *content_id)
 {
 	g_return_if_fail (mime_part != NULL);
 	
@@ -198,7 +207,7 @@ g_mime_part_set_content_id (GMimePart *mime_part, const gchar *content_id)
  *
  * Returns the content id for the specified mime part.
  **/
-const gchar *
+const char *
 g_mime_part_get_content_id (GMimePart *mime_part)
 {
 	g_return_val_if_fail (mime_part != NULL, NULL);
@@ -215,7 +224,7 @@ g_mime_part_get_content_id (GMimePart *mime_part)
  * Set the content md5 for the specified mime part.
  **/
 void
-g_mime_part_set_content_md5 (GMimePart *mime_part, const gchar *content_md5)
+g_mime_part_set_content_md5 (GMimePart *mime_part, const char *content_md5)
 {
 	const GMimeContentType *type;
 	
@@ -232,11 +241,28 @@ g_mime_part_set_content_md5 (GMimePart *mime_part, const gchar *content_md5)
 	
 	if (content_md5) {
 		mime_part->content_md5 = g_strdup (content_md5);
-	} else if (mime_part->content && mime_part->content->len) {
+	} else if (mime_part->content && mime_part->content->stream) {
 		char digest[16], b64digest[32];
 		int len, state, save;
+		GMimeStream *stream;
+		GByteArray *buf;
 		
-		md5_get_digest (mime_part->content->data, mime_part->content->len, digest);
+		stream = mime_part->content->stream;
+		if (!GMIME_IS_STREAM_MEM (stream) || NEEDS_DECODING (mime_part->content->encoding)) {
+			stream = g_mime_stream_mem_new ();
+			g_mime_data_wrapper_write_to_stream (mime_part->content, stream);
+		} else {
+			stream = mime_part->content->stream;
+			g_mime_stream_ref (stream);
+		}
+		
+		buf = GMIME_STREAM_MEM (stream)->buffer;
+		
+		md5_get_digest (buf->data + stream->bound_start,
+				g_mime_stream_length (stream),
+				digest);
+		
+		g_mime_stream_unref (stream);
 		
 		state = save = 0;
 		len = g_mime_utils_base64_encode_close (digest, 16, b64digest, &state, &save);
@@ -261,11 +287,32 @@ g_mime_part_verify_content_md5 (GMimePart *mime_part)
 {
 	char digest[16], b64digest[32];
 	int len, state, save;
+	GMimeStream *stream;
+	GByteArray *buf;
 	
 	g_return_val_if_fail (mime_part != NULL, FALSE);
+	g_return_val_if_fail (mime_part->content != NULL, FALSE);
 	g_return_val_if_fail (mime_part->content_md5 != NULL, FALSE);
 	
-	md5_get_digest (mime_part->content->data, mime_part->content->len, digest);
+	stream = mime_part->content->stream;
+	if (!stream)
+		return FALSE;
+	
+	if (!GMIME_IS_STREAM_MEM (stream) || NEEDS_DECODING (mime_part->content->encoding)) {
+		stream = g_mime_stream_mem_new ();
+		g_mime_data_wrapper_write_to_stream (mime_part->content, stream);
+	} else {
+		stream = mime_part->content->stream;
+		g_mime_stream_ref (stream);
+	}
+	
+	buf = GMIME_STREAM_MEM (stream)->buffer;
+	
+	md5_get_digest (buf->data + stream->bound_start,
+			g_mime_stream_length (stream),
+			digest);
+	
+	g_mime_stream_unref (GMIME_STREAM (stream));
 	
 	state = save = 0;
 	len = g_mime_utils_base64_encode_close (digest, 16, b64digest, &state, &save);
@@ -281,7 +328,7 @@ g_mime_part_verify_content_md5 (GMimePart *mime_part)
  *
  * Returns the content md5 for the specified mime part.
  **/
-const gchar *
+const char *
 g_mime_part_get_content_md5 (GMimePart *mime_part)
 {
 	g_return_val_if_fail (mime_part != NULL, NULL);
@@ -298,7 +345,7 @@ g_mime_part_get_content_md5 (GMimePart *mime_part)
  * Set the content location for the specified mime part.
  **/
 void
-g_mime_part_set_content_location (GMimePart *mime_part, const gchar *content_location)
+g_mime_part_set_content_location (GMimePart *mime_part, const char *content_location)
 {
 	g_return_if_fail (mime_part != NULL);
 	
@@ -315,7 +362,7 @@ g_mime_part_set_content_location (GMimePart *mime_part, const gchar *content_loc
  *
  * Returns the content location for the specified mime part.
  **/
-const gchar *
+const char *
 g_mime_part_get_content_location (GMimePart *mime_part)
 {
 	g_return_val_if_fail (mime_part != NULL, NULL);
@@ -405,7 +452,7 @@ g_mime_part_get_encoding (GMimePart *mime_part)
  * GMIME_PART_ENCODING_7BIT, GMIME_PART_ENCODING_8BIT,
  * GMIME_PART_ENCODING_BASE64 and GMIME_PART_ENCODING_QUOTEDPRINTABLE.
  **/
-const gchar *
+const char *
 g_mime_part_encoding_to_string (GMimePartEncodingType encoding)
 {
 	switch (encoding) {
@@ -435,7 +482,7 @@ g_mime_part_encoding_to_string (GMimePartEncodingType encoding)
  * GMIME_PART_ENCODING_QUOTEDPRINTABLE.
  **/
 GMimePartEncodingType
-g_mime_part_encoding_from_string (const gchar *encoding)
+g_mime_part_encoding_from_string (const char *encoding)
 {
 	if (!g_strcasecmp (encoding, "7bit"))
 		return GMIME_PART_ENCODING_7BIT;
@@ -457,7 +504,7 @@ g_mime_part_encoding_from_string (const gchar *encoding)
  * Set the content disposition for the specified mime part
  **/
 void
-g_mime_part_set_content_disposition (GMimePart *mime_part, const gchar *disposition)
+g_mime_part_set_content_disposition (GMimePart *mime_part, const char *disposition)
 {
 	g_return_if_fail (mime_part != NULL);
 	
@@ -480,7 +527,7 @@ g_mime_part_set_content_disposition (GMimePart *mime_part, const gchar *disposit
  *
  * Returns the content disposition for the specified mime part.
  **/
-const gchar *
+const char *
 g_mime_part_get_content_disposition (GMimePart *mime_part)
 {
 	g_return_val_if_fail (mime_part != NULL, NULL);
@@ -501,7 +548,7 @@ g_mime_part_get_content_disposition (GMimePart *mime_part)
  * Add a content-disposition parameter to the specified mime part.
  **/
 void
-g_mime_part_add_content_disposition_parameter (GMimePart *mime_part, const gchar *name, const gchar *value)
+g_mime_part_add_content_disposition_parameter (GMimePart *mime_part, const char *name, const char *value)
 {
 	GMimeParam *param;
 	
@@ -537,8 +584,8 @@ g_mime_part_add_content_disposition_parameter (GMimePart *mime_part, const gchar
  * Returns the value of a previously defined content-disposition
  * parameter specified by #name.
  **/
-const gchar *
-g_mime_part_get_content_disposition_parameter (GMimePart *mime_part, const gchar *name)
+const char *
+g_mime_part_get_content_disposition_parameter (GMimePart *mime_part, const char *name)
 {
 	GMimeParam *param;
 	
@@ -562,7 +609,7 @@ g_mime_part_get_content_disposition_parameter (GMimePart *mime_part, const gchar
  * "name" parameter on the Content-Type.
  **/
 void
-g_mime_part_set_filename (GMimePart *mime_part, const gchar *filename)
+g_mime_part_set_filename (GMimePart *mime_part, const char *filename)
 {
 	GMimeParam *param;
 	
@@ -600,17 +647,15 @@ g_mime_part_set_filename (GMimePart *mime_part, const gchar *filename)
  * see if the "filename" parameter was set on the Content-Disposition
  * and if not then checks the "name" parameter in the Content-Type.
  **/
-const gchar *
+const char *
 g_mime_part_get_filename (const GMimePart *mime_part)
 {
-	GMimeParam *param;
+	GMimeParam *param = NULL;
 	
 	g_return_val_if_fail (mime_part != NULL, NULL);
 	
-	if (!mime_part->disposition || !mime_part->disposition->param_hash)
-		return NULL;
-	
-	param = g_hash_table_lookup (mime_part->disposition->param_hash, "filename");
+	if (mime_part->disposition && mime_part->disposition->param_hash)
+		param = g_hash_table_lookup (mime_part->disposition->param_hash, "filename");
 	
 	if (!param || !param->value) {
 		/* check the "name" param in the content-type */
@@ -622,7 +667,7 @@ g_mime_part_get_filename (const GMimePart *mime_part)
 
 
 static void
-read_random_pool (gchar *buffer, size_t bytes)
+read_random_pool (char *buffer, size_t bytes)
 {
 	int fd;
 	
@@ -646,9 +691,9 @@ read_random_pool (gchar *buffer, size_t bytes)
  * Sets the boundary on the mime part.
  **/
 void
-g_mime_part_set_boundary (GMimePart *mime_part, const gchar *boundary)
+g_mime_part_set_boundary (GMimePart *mime_part, const char *boundary)
 {
-	gchar bbuf[27];
+	char bbuf[27];
 	
 	g_return_if_fail (mime_part != NULL);
 	
@@ -678,7 +723,7 @@ g_mime_part_set_boundary (GMimePart *mime_part, const gchar *boundary)
  *
  * Returns the boundary on the mime part.
  **/
-const gchar *
+const char *
 g_mime_part_get_boundary (GMimePart *mime_part)
 {
 	g_return_val_if_fail (mime_part != NULL, NULL);
@@ -696,15 +741,19 @@ g_mime_part_get_boundary (GMimePart *mime_part)
  * Sets the content of the Mime Part (only non-multiparts)
  **/
 void
-g_mime_part_set_content (GMimePart *mime_part, const gchar *content, guint len)
+g_mime_part_set_content (GMimePart *mime_part, const char *content, guint len)
 {
+	GMimeStream *stream;
+	
 	g_return_if_fail (mime_part != NULL);
 	
-	if (mime_part->content)
-		g_byte_array_free (mime_part->content, TRUE);
+	if (!mime_part->content)
+		mime_part->content = g_mime_data_wrapper_new ();
 	
-	mime_part->content = g_byte_array_new ();
-	g_byte_array_append (mime_part->content, content, len);
+	stream = g_mime_stream_mem_new_with_buffer (content, len);
+	g_mime_data_wrapper_set_stream (mime_part->content, stream);
+	g_mime_data_wrapper_set_encoding (mime_part->content, GMIME_PART_ENCODING_DEFAULT);
+	g_mime_stream_unref (stream);
 }
 
 
@@ -718,70 +767,17 @@ g_mime_part_set_content (GMimePart *mime_part, const gchar *content, guint len)
 void
 g_mime_part_set_content_byte_array (GMimePart *mime_part, GByteArray *content)
 {
+	GMimeStream *stream;
+	
 	g_return_if_fail (mime_part != NULL);
 	
-	if (mime_part->content)
-		g_byte_array_free (mime_part->content, TRUE);
+	if (!mime_part->content)
+		mime_part->content = g_mime_data_wrapper_new ();
 	
-	mime_part->content = content;
-}
-
-void
-g_mime_part_append_pre_encoded_content (GMimePart              * part,
-                                        const gchar            * content,
-                                        guint                    len,
-                                        GMimePartEncodingType    encoding)
-{
-	gchar * raw;
-	gint old_size = 0;
-	gint max_new_size = 0;
-
-	g_return_if_fail (part != NULL);
-	g_return_if_fail (content != NULL);
-	
-	/* make sure we've got a content byte array */
-	if (part->content == NULL)
-		part->content = g_byte_array_new ();
-
-	/* make sure we've got enough room for the new decoded data */
-	old_size = part->content->len;
-	max_new_size = old_size + len;
-	g_byte_array_set_size (part->content, max_new_size);
-
-	/* write the decoded data */
-	raw = part->content->data + old_size;
-	switch (encoding)
-	{
-		case GMIME_PART_ENCODING_BASE64:
-			len = g_mime_utils_base64_decode_step (content, len, raw, &part->append_state, &part->append_save);
-			g_byte_array_set_size (part->content, old_size+len);
-			break;
-
-		case GMIME_PART_ENCODING_QUOTEDPRINTABLE:
-			len = g_mime_utils_quoted_decode_step (content, len, raw, &part->append_state, &part->append_save);
-			g_byte_array_set_size (part->content, old_size+len);
-			break;
-
-		default:
-			memcpy (raw, content, len);
-
-			/* do some smart 8bit detection */
-			if (encoding == GMIME_PART_ENCODING_DEFAULT)
-			{
-				/* if we were at default before and this is 8bit text, set encoding to 8bit */
-				if (part->encoding==GMIME_PART_ENCODING_DEFAULT
-					&& g_mime_utils_text_is_8bit (raw, len))
-					encoding = GMIME_PART_ENCODING_8BIT;
-
-				/* if we guessed 8bit before but this isn't an 8bit piece, set encoding back to default */
-				else if (part->encoding == GMIME_PART_ENCODING_8BIT)
-					encoding = g_mime_utils_text_is_8bit (raw, len)
-						? GMIME_PART_ENCODING_8BIT
-						: GMIME_PART_ENCODING_DEFAULT;
-			}
-	}
-
-	part->encoding = encoding;
+	stream = g_mime_stream_mem_new_with_byte_array (content);
+	g_mime_data_wrapper_set_stream (mime_part->content, stream);
+	g_mime_data_wrapper_set_encoding (mime_part->content, GMIME_PART_ENCODING_DEFAULT);
+	g_mime_stream_unref (stream);
 }
 
 
@@ -795,61 +791,140 @@ g_mime_part_append_pre_encoded_content (GMimePart              * part,
  * Sets the encoding type and raw content on the mime part after decoding the content.
  **/
 void
-g_mime_part_set_pre_encoded_content (GMimePart *mime_part, const gchar *content,
+g_mime_part_set_pre_encoded_content (GMimePart *mime_part, const char *content,
 				     guint len, GMimePartEncodingType encoding)
 {
-	gchar *raw;
-	gint save = 0, state = 0;
+	GMimeStream *stream, *filtered_stream;
+	GMimeFilter *filter;
 	
 	g_return_if_fail (mime_part != NULL);
 	g_return_if_fail (content != NULL);
 	
-	if (mime_part->content)
-		g_byte_array_free (mime_part->content, TRUE);
+	if (!mime_part->content)
+		mime_part->content = g_mime_data_wrapper_new ();
 	
-	mime_part->content = g_byte_array_new ();
-	g_byte_array_set_size (mime_part->content, len);
-	raw = mime_part->content->data;
+	stream = g_mime_stream_mem_new ();
+	filtered_stream = g_mime_stream_filter_new_with_stream (stream);
 	switch (encoding) {
 	case GMIME_PART_ENCODING_BASE64:
-		len = g_mime_utils_base64_decode_step (content, len, raw, &state, &save);
-		g_byte_array_set_size (mime_part->content, len);
+		filter = g_mime_filter_basic_new_type (GMIME_FILTER_BASIC_BASE64_DEC);
+		g_mime_stream_filter_add (GMIME_STREAM_FILTER (filtered_stream), filter);
 		break;
 	case GMIME_PART_ENCODING_QUOTEDPRINTABLE:
-		len = g_mime_utils_quoted_decode_step (content, len, raw, &state, &save);
-		g_byte_array_set_size (mime_part->content, len);
+		filter = g_mime_filter_basic_new_type (GMIME_FILTER_BASIC_QP_DEC);
+		g_mime_stream_filter_add (GMIME_STREAM_FILTER (filtered_stream), filter);
 		break;
 	default:
-		memcpy (raw, content, len);
-		
-		/* do some smart 8bit detection */
-		if (encoding == GMIME_PART_ENCODING_DEFAULT && g_mime_utils_text_is_8bit (raw, len))
-			encoding = GMIME_PART_ENCODING_8BIT;
+		break;
 	}
 	
+	g_mime_stream_write (filtered_stream, (char *) content, len);
+	g_mime_stream_flush (filtered_stream);
+	g_mime_stream_unref (filtered_stream);
+	
+	g_mime_stream_reset (stream);
+	g_mime_data_wrapper_set_stream (mime_part->content, stream);
+	g_mime_data_wrapper_set_encoding (mime_part->content, GMIME_PART_ENCODING_DEFAULT);
+	g_mime_stream_unref (stream);
+	
 	mime_part->encoding = encoding;
 }
 
 
 /**
- * g_mime_part_get_content: 
- * @mime_part: the GMimePart to be decoded.
- * @len: decoded length (to be set after processing)
- * 
- * Returns a gchar * pointer to the raw contents of the MIME Part
- * and sets %len to the length of the buffer.
+ * g_mime_part_set_content_object:
+ * @mime_part: MIME Part
+ * @content: content object
+ *
+ * Sets the content object on the mime part.
  **/
-const gchar *
-g_mime_part_get_content (const GMimePart *mime_part, guint *len)
+void
+g_mime_part_set_content_object (GMimePart *mime_part, GMimeDataWrapper *content)
+{
+	g_return_if_fail (mime_part != NULL);
+	
+	if (mime_part->content)
+		g_mime_data_wrapper_destroy (mime_part->content);
+	
+	mime_part->content = content;
+}
+
+
+
+/**
+ * g_mime_part_get_content_object:
+ * @mime_part: MIME part object
+ *
+ * Returns the data-wrapper for the mime part's contents.
+ **/
+const GMimeDataWrapper *
+g_mime_part_get_content_object (const GMimePart *mime_part)
 {
 	g_return_val_if_fail (mime_part != NULL, NULL);
 	
-	if (!mime_part->content)
+	return mime_part->content;
+}
+
+
+/**
+ * g_mime_part_get_content: 
+ * @mime_part: MIME part object
+ * @len: pointer to the content length
+ * 
+ * Returns a char * pointer to the raw contents of the MIME Part
+ * and sets %len to the length of the buffer.
+ **/
+const char *
+g_mime_part_get_content (const GMimePart *mime_part, guint *len)
+{
+	GMimeStream * stream;
+	const gchar * retval = NULL;
+	
+	g_return_val_if_fail (mime_part != NULL, NULL);
+	
+	if (!mime_part->content || !mime_part->content->stream) {
+		g_warning ("no content set on this mime part");
 		return NULL;
+	}
 	
-	*len = mime_part->content->len;
+	stream = mime_part->content->stream;
+	if (!GMIME_IS_STREAM_MEM (stream) || NEEDS_DECODING (mime_part->content->encoding))
+	{
+		/* Decode and cache this mime part's contents... */
+		GMimeStream *cache;
+		
+		GByteArray * buf = g_byte_array_new ();
+		cache = g_mime_stream_mem_new_with_byte_array (buf);
+		
+		g_mime_data_wrapper_write_to_stream (mime_part->content, cache);
+		
+		g_mime_data_wrapper_set_stream (mime_part->content, cache);
+		g_mime_data_wrapper_set_encoding (mime_part->content, GMIME_PART_ENCODING_DEFAULT);
+		g_mime_stream_unref (cache);
+
+		*len = buf->len;
+		retval = buf->data;
+	}
+	else
+	{
+		GByteArray * buf = GMIME_STREAM_MEM (stream)->buffer;
+		gint start_index = 0;
+		gint end_index = buf->len;
+
+		/* check boundaries */
+		if (stream->bound_start>=0)
+			start_index = CLAMP (stream->bound_start, 0, buf->len);
+		if (stream->bound_end>=0)
+			end_index = CLAMP (stream->bound_end, 0, buf->len);
+		if (end_index < start_index)
+			end_index = start_index;
+
+		*len = end_index - start_index;
+		retval = buf->data + start_index;
+	}
 	
-	return mime_part->content->data;
+	
+	return retval;
 }
 
 
@@ -874,12 +949,12 @@ g_mime_part_add_subpart (GMimePart *mime_part, GMimePart *subpart)
 		mime_part->children = g_list_append (mime_part->children, subpart);
 }
 
-static gchar *
+static char *
 get_content_disposition (GMimePart *mime_part)
 {
 	GString *string;
 	GList *params;
-	gchar *str;
+	char *str;
 	
 	if (!mime_part->disposition)
 		return NULL;
@@ -896,7 +971,7 @@ get_content_disposition (GMimePart *mime_part)
 	
 	while (params) {
 		GMimeParam *param;
-		gchar *buf;
+		char *buf;
 		
 		param = params->data;
 		buf = g_mime_param_to_string (param);
@@ -918,12 +993,12 @@ get_content_disposition (GMimePart *mime_part)
 }
 
 
-static gchar *
+static char *
 get_content_type (GMimeContentType *mime_type)
 {
 	GString *string;
 	GList *params;
-	gchar *str, *type;
+	char *str, *type;
 	
 	g_return_val_if_fail (mime_type != NULL, NULL);
 	
@@ -938,7 +1013,7 @@ get_content_type (GMimeContentType *mime_type)
 	
 	while (params) {
 		GMimeParam *param;
-		gchar *buf;
+		char *buf;
 		
 		param = params->data;
 		buf = g_mime_param_to_string (param);
@@ -962,56 +1037,56 @@ get_content_type (GMimeContentType *mime_type)
 	return type;
 }
 
-static gchar *
-get_content (GMimePart *part)
+static void
+write_content (GMimePart *part, GMimeStream *stream)
 {
-	gchar *content;
-	gint save = 0, state = 0;
-	gint len;
+	GMimeStream *filtered_stream;
+	GMimeFilter *filter;
+	ssize_t written;
 	
-	if (!part->content)
-		return g_strdup ("");
+	if (!part->content || !part->content->stream)
+		return;
 	
+	filtered_stream = g_mime_stream_filter_new_with_stream (stream);
 	switch (part->encoding) {
 	case GMIME_PART_ENCODING_BASE64:
-		content = g_malloc (BASE64_ENCODE_LEN (part->content->len));
-		len = g_mime_utils_base64_encode_close (part->content->data, part->content->len, content, &state, &save);
-		content[len] = '\0';
+		filter = g_mime_filter_basic_new_type (GMIME_FILTER_BASIC_BASE64_ENC);
+		g_mime_stream_filter_add (GMIME_STREAM_FILTER (filtered_stream), filter);
 		break;
 	case GMIME_PART_ENCODING_QUOTEDPRINTABLE:
-		state = -1;
-		content = g_malloc (QP_ENCODE_LEN (part->content->len));
-		len = g_mime_utils_quoted_encode_close (part->content->data, part->content->len, content, &state, &save);
-		content[len] = '\0';
+		filter = g_mime_filter_basic_new_type (GMIME_FILTER_BASIC_QP_ENC);
+		g_mime_stream_filter_add (GMIME_STREAM_FILTER (filtered_stream), filter);
 		break;
 	default:
-		content = g_strndup (part->content->data, part->content->len);
+		break;
 	}
 	
-	return content;
+	written = g_mime_data_wrapper_write_to_stream (part->content, filtered_stream);
+	g_mime_stream_flush (filtered_stream);
+	g_mime_stream_unref (filtered_stream);
+	
+	/* this is just so that I get a warning on fail... */
+	g_return_if_fail (written != -1);
 }
 
+
 /**
- * g_mime_part_to_string: Write the MIME Part to a string
+ * g_mime_part_write_to_stream: Write the MIME Part to a string
  * @mime_part: MIME Part
- * @toplevel: mime part is the root mime part
+ * @stream: output stream
  *
- * Returns an allocated string containing the MIME Part. If toplevel
- * is set to TRUE, then the MIME Part header will contain needed MIME
- * headers for rfc822 messages.
+ * Writes the contents of the MIME Part to @stream.
  **/
-gchar *
-g_mime_part_to_string (GMimePart *mime_part, gboolean toplevel)
+void
+g_mime_part_write_to_stream (GMimePart *mime_part, GMimeStream *stream)
 {
-	gchar *str;
-	
-	g_return_val_if_fail (mime_part != NULL, NULL);
+	g_return_if_fail (mime_part != NULL);
+	g_return_if_fail (stream != NULL);
 	
 	if (g_mime_content_type_is_type (mime_part->mime_type, "multipart", "*")) {
-		const gchar *boundary;
-		GString *contents;
+		const char *boundary;
 		GList *child;
-		gchar *content_type;
+		char *content_type;
 		
 		/* make sure there's a boundary, else force a random boundary */
 		boundary = g_mime_part_get_boundary (mime_part);
@@ -1021,73 +1096,41 @@ g_mime_part_to_string (GMimePart *mime_part, gboolean toplevel)
 		}
 		
 		content_type = get_content_type (mime_part->mime_type);
-		
-		contents = g_string_new ("");
+		g_mime_stream_printf (stream, "%s\n", content_type);		
+		g_free (content_type);
 		
 		child = mime_part->children;
 		while (child) {
-			gchar *part, *mime_string;
-			
-			mime_string = g_mime_part_to_string (child->data, FALSE);
-			
-			part = g_strdup_printf ("--%s\n%s\n", boundary, mime_string);
-			
-			g_string_append (contents, part);
-			
-			g_free (mime_string);
-			g_free (part);
+			g_mime_stream_printf (stream, "--%s\n", boundary);
+			g_mime_part_write_to_stream (child->data, stream);
+			g_mime_stream_write (stream, "\n", 1);
 			
 			child = child->next;
 		}
 		
-		if (toplevel) {
-			/* add message header stuff first */
-			str = g_strdup_printf ("MIME-Version: 1.0\n"
-					       "%s\n"  /* content-type */
-					       "This is a multi-part message in MIME format.\n\n"
-					       "%s\n--%s--\n",
-					       content_type,
-					       contents->str,
-					       boundary);
-		} else {
-			str = g_strdup_printf ("%s\n"  /* content-type */
-					       "%s\n--%s--\n",
-					       content_type,
-					       contents->str,
-					       boundary);
-		}
-		
-		g_free (content_type);
-		g_string_free (contents, TRUE);
+		g_mime_stream_printf (stream, "\n--%s--\n", boundary);
 	} else {
-		gchar *content_type;
-		gchar *disposition;
-		gchar *description;
-		gchar *content_id;
-		gchar *content_md5;
-		gchar *content_location;
-		gchar *content;
-		gchar *text;
-		GString *string;
-		
-		string = g_string_new ("");
-		
-		if (toplevel)
-			g_string_append (string, "MIME-Version: 1.0\n");
+		char *content_type;
+		char *disposition;
+		char *description;
+		char *content_id;
+		char *content_md5;
+		char *content_location;
+		char *text;
 		
 		/* Content-Type: */
 		content_type = get_content_type (mime_part->mime_type);
-		g_string_append (string, content_type);
+		g_mime_stream_write_string (stream, content_type);
 		g_free (content_type);
 		
 		/* Content-Transfer-Encoding: */
 		if (mime_part->encoding != GMIME_PART_ENCODING_DEFAULT) {
-			gchar *content_encoding;
+			char *content_encoding;
 			
 			content_encoding = g_strdup_printf ("Content-Transfer-Encoding: %s\n",
 							    g_mime_part_encoding_to_string (mime_part->encoding));
 			
-			g_string_append (string, content_encoding);
+			g_mime_stream_write_string (stream, content_encoding);
 			g_free (content_encoding);
 		}
 		
@@ -1097,7 +1140,7 @@ g_mime_part_to_string (GMimePart *mime_part, gboolean toplevel)
 			text = g_mime_utils_header_printf ("Content-Disposition: %s\n",
 							   disposition);
 			g_free (disposition);
-			g_string_append (string, text);
+			g_mime_stream_write_string (stream, text);
 			g_free (text);
 		}
 		
@@ -1107,43 +1150,62 @@ g_mime_part_to_string (GMimePart *mime_part, gboolean toplevel)
 			description = g_mime_utils_header_printf ("Content-Description: %s\n", text);
 			g_free (text);
 			
-			g_string_append (string, description);
+			g_mime_stream_write_string (stream, description);
 			g_free (description);
 		}
 		
 		/* Content-Location: */
 		if (mime_part->content_location) {
-			content_location = g_strdup_printf ("Content-Location: %s\n", mime_part->content_location);
-			g_string_append (string, content_location);
+			content_location = g_strdup_printf ("Content-Location: %s\n",
+							    mime_part->content_location);
+			g_mime_stream_write_string (stream, content_location);
 			g_free (content_location);
 		}
 		
 		/* Content-Md5: */
 		if (mime_part->content_md5) {
 			content_md5 = g_strdup_printf ("Content-MD5: %s\n", mime_part->content_md5);
-			g_string_append (string, content_md5);
+			g_mime_stream_write_string (stream, content_md5);
 			g_free (content_md5);
 		}
 		
 		/* Content-Id: */
 		if (mime_part->content_id) {
 			content_id = g_strdup_printf ("Content-Id: %s\n", mime_part->content_id);
-			g_string_append (string, content_id);
+			g_mime_stream_write_string (stream, content_id);
 			g_free (content_id);
 		}
 		
-		g_string_append_c (string, '\n');
-		
-		content = get_content (mime_part);
-		
-		g_string_append (string, content);
-		g_string_append_c (string, '\n');
-		
-		g_free (content);
-		
-		str = string->str;
-		g_string_free (string, FALSE);
+		g_mime_stream_write (stream, "\n", 1);
+		write_content (mime_part, stream);
+		g_mime_stream_write (stream, "\n", 1);
 	}
+}
+
+
+/**
+ * g_mime_part_to_string: Write the MIME Part to a string
+ * @mime_part: MIME Part
+ *
+ * Returns an allocated string containing the MIME Part.
+ **/
+char *
+g_mime_part_to_string (GMimePart *mime_part)
+{
+	GMimeStream *stream;
+	GByteArray *buf;
+	char *str;
+	
+	g_return_val_if_fail (mime_part != NULL, NULL);
+	
+	buf = g_byte_array_new ();
+	stream = g_mime_stream_mem_new ();
+	g_mime_stream_mem_set_byte_array (GMIME_STREAM_MEM (stream), buf);
+	g_mime_part_write_to_stream (mime_part, stream);
+	g_mime_stream_unref (stream);
+	g_byte_array_append (buf, "", 1);
+	str = buf->data;
+	g_byte_array_free (buf, FALSE);
 	
 	return str;
 }
@@ -1186,7 +1248,7 @@ g_mime_part_foreach (GMimePart *mime_part, GMimePartFunc callback, gpointer data
  * or NULL if a match cannot be found.
  **/
 const GMimePart *
-g_mime_part_get_subpart_from_content_id (GMimePart *mime_part, const gchar *content_id)
+g_mime_part_get_subpart_from_content_id (GMimePart *mime_part, const char *content_id)
 {
 	GList *child;
 	
