@@ -197,6 +197,76 @@ compare_ppA_to_ppA_by_linecount (const void* va, const void* vb)
 	return a_loc - b_loc;
 } 
 
+static int
+compare_ppA_to_ppA_by_action (const void * va, const void * vb)
+{
+	const Article * a = *(const Article **)va;
+	const Article * b = *(const Article **)vb;
+	gint ia, ib;
+
+	ia = article_flag_on (a, STATE_SAVE_QUEUED) ? 1 : 0;
+	ib = article_flag_on (b, STATE_SAVE_QUEUED) ? 1 : 0;
+	if (ia != ib)
+		return ib - ia;
+
+	ia = article_flag_on (a, STATE_DOWNLOAD_FLAGGED) ? 1 : 0;
+	ib = article_flag_on (b, STATE_DOWNLOAD_FLAGGED) ? 1 : 0;
+	if (ia != ib)
+		return ib - ia;
+
+	ia = article_flag_on (a, STATE_DECODED) ? 1 : 0;
+	ib = article_flag_on (b, STATE_DECODED) ? 1 : 0;
+	if (ia != ib)
+		return ib - ia;
+
+	ia = article_has_body (a) ? 1 : 0;
+	ib = article_has_body (b) ? 1 : 0;
+	if (ia != ib)
+		return ib - ia;
+
+	return 0;
+}
+
+static int
+compare_ppA_to_ppA_by_read (const void * va, const void * vb)
+{
+	const Article * a = *(const Article **)va;
+	const Article * b = *(const Article **)vb;
+	const gboolean a_is_read = article_is_read (a);
+	const gboolean b_is_read = article_is_read (b);
+	const gboolean a_is_new = article_is_new (a);
+	const gboolean b_is_new = article_is_new (b);
+	gint ia, ib;
+
+	ia = (a_is_new?1:0) + a->new_children;
+	ib = (b_is_new?1:0) + b->new_children;
+	if (ia != ib)
+		return ib - ia;
+
+	ia = (a_is_read?0:1) + a->unread_children;
+	ib = (b_is_read?0:1) + b->unread_children;
+	if (ia != ib)
+		return ib - ia;
+
+	ia = a_is_read ? 0 : 1;
+	ib = b_is_read ? 0 : 1;
+	if (ia != ib)
+		return ib - ia;
+
+	ia = article_flag_on (a, STATE_MULTIPART_ALL) ? 1 : 0;
+	ib = article_flag_on (b, STATE_MULTIPART_ALL) ? 1 : 0;
+	if (ia != ib)
+		return ib - ia;
+
+	ia = article_flag_on (a, STATE_MULTIPART_SOME) ? 1 : 0;
+	ib = article_flag_on (b, STATE_MULTIPART_SOME) ? 1 : 0;
+	if (ia != ib)
+		return ib - ia;
+
+	return 0;
+}
+
+
 static int
 compare_ppA_to_ppA_by_date (const void* va, const void* vb)
 {
@@ -207,8 +277,8 @@ compare_ppA_to_ppA_by_date (const void* va, const void* vb)
 static int
 compare_ppA_to_ppA_by_message_id (const void* a, const void* b)
 {
-	const gchar * msg_id_a = (**(const Article **)a).message_id;
-	const gchar * msg_id_b = (**(const Article **)b).message_id;
+	const gchar * msg_id_a = article_get_message_id (*(const Article**)a);
+	const gchar * msg_id_b = article_get_message_id (*(const Article**)b);
 	return strcmp (msg_id_a, msg_id_b);
 }
 
@@ -241,7 +311,9 @@ sort_articles (Article      ** buf,
 			ArticleStruct * as = g_new (ArticleStruct, article_qty);
 			for (i=0; i<article_qty; ++i)
 			{
-				as[i].data = article_get_short_author_str (buf[i]);
+				gchar author[256];
+				article_get_short_author_str (buf[i], author, sizeof(author));
+				as[i].data = g_strdup (author);
 				as[i].article = buf[i];
 				g_strdown (as[i].data);
 			}
@@ -271,6 +343,16 @@ sort_articles (Article      ** buf,
 			msort (buf, article_qty, sizeof(Article*), compare_ppA_to_ppA_by_message_id);
 			break;
 		}
+		case ARTICLE_SORT_ACTION_STATE:
+		{
+			msort (buf, article_qty, sizeof(Article*), compare_ppA_to_ppA_by_action);
+			break;
+		}
+		case ARTICLE_SORT_READ_STATE:
+		{
+			msort (buf, article_qty, sizeof(Article*), compare_ppA_to_ppA_by_read);
+			break;
+		}
 		case ARTICLE_SORT_SUBJECT:
 		default:
 		{
@@ -342,7 +424,7 @@ thread_articles (GPtrArray    * articles,
 
 	if (status != NULL)
 		status_item_emit_status_va (status, _("Threading %u articles"), qty);
-
+	/* make a plausiably-legal article FIXME: indent this right after the string freeze is over*/ search_a.number = 1; search_a.subject = "dummy subject"; search_a.references = NULL;
 	/* unthread the articles, just in case they were threaded before */
 	for (i=0; i!=qty; ++i) {
 		Article * a = ARTICLE(g_ptr_array_index(articles,i));
