@@ -4,14 +4,12 @@
 #include <glib.h>
 
 extern gchar      * data_dir;
-extern gchar      * temp_dir;
 extern gchar      * download_dir;
 extern gint         acache_max_megs;
 extern gboolean     acache_flush_on_exit;
 extern gboolean     break_thread_when_subject_changes;
 
 void base_prefs_init (const gchar * data_dir,
-                      const gchar * temp_dir,
                       const gchar * download_dir,
                       gint          acache_max_megs,
                       gboolean      acache_flush_on_exit,
