@@ -28,7 +28,7 @@
 #include <string.h>
 #include <time.h>
 #include <unistd.h>
-#include <sys/timeb.h>
+#include <sys/time.h>
 
 #include <ctype.h>
 #include <stdio.h>
@@ -43,46 +43,10 @@
 #include <pan/base/pan-i18n.h>
 #include <pan/base/pan-glib-extensions.h>
 
-/*********************
-**********************  Defines / Enumerated types
-*********************/
-
-/*********************
-**********************  Macros
-*********************/
-
-/*********************
-**********************  Structures / Typedefs
-*********************/
-
-/*********************
-**********************  Private Function Prototypes
-*********************/
-
-/*********************
-**********************  Variables
-*********************/
-
-/***********
-************  Extern
-***********/
-
-/***********
-************  Public
-***********/
-
-/***********
-************  Private
-***********/
-
 /*********************
 **********************  BEGINNING OF SOURCE
 *********************/
 
-/************
-*************  PRIVATE
-************/
-
 /*****
 ******
 *****/
@@ -917,18 +881,18 @@ gnksa_generate_message_id (const gchar * domain_name)
 	/* add unique local part to message-id */
 	{
 		gulong pid;
-		struct timeb now;
+		struct timeval now;
 		struct tm local_now;
 		char buf[64];
 
 		pid = (gulong) getpid ();
-		ftime (&now);
-		localtime_r (&now.time, &local_now);
+		gettimeofday (&now, NULL);
+		localtime_r (&now.tv_sec, &local_now);
 		strftime (buf, sizeof(buf), "%Y.%m.%d.%H.%M.%S", &local_now);
 
 		g_string_append (msgid, "pan.");
 		g_string_append (msgid, buf);
-		g_string_sprintfa (msgid, ".%hu.%lu", now.millitm, pid);
+		g_string_sprintfa (msgid, ".%ld.%lu", now.tv_usec, pid);
 	}
 
 	/* delimit */
