@@ -20,9 +20,10 @@
  */
 
 #include <config.h>
-#include <gnome.h>
-#include "dialogs.h"
-#include "pan-about.h"
+#include <libgnomeui/libgnomeui.h>
+#include <pan/base/pan-i18n.h>
+#include <pan/dialogs/dialogs.h>
+#include <pan/dialogs/pan-about.h>
 
 static gchar * get_credits_code (void);
 static gchar * get_credits_users (void);
