@@ -24,9 +24,10 @@
 
 #include <string.h>
 
-#include <gnome.h>
+#include <libgnomeui/libgnomeui.h>
 
 #include <pan/base/debug.h>
+#include <pan/base/pan-i18n.h>
 #include <pan/base/pan-glib-extensions.h>
 
 #include <pan/globals.h>
@@ -46,12 +47,8 @@ extern GtkTooltips * ttips;
 static void
 showmore_cb (GtkButton *button, gpointer user_data)
 {
-	const gchar* showme = NULL;
-
-	pan_lock ();
-	showme = (const gchar*) gtk_object_get_data ( GTK_OBJECT(button), "text" );
+	const gchar* showme = (const gchar*) gtk_object_get_data (GTK_OBJECT(button), "text");
 	gnome_ok_dialog_parented (showme, GTK_WINDOW(Pan.window));
-	pan_unlock();
 }
 
 static gint
@@ -92,7 +89,7 @@ add_header_nolock (GtkWidget        * table,
 	           const gchar      * str1,
 	           const gchar      * str2,
 	           gchar              delimiter,
-                   gint               max_width )
+                   gint               max_width)
 {
 	GtkTable * t;
 	GtkWidget * w1;
@@ -165,10 +162,9 @@ add_header_nolock (GtkWidget        * table,
 }
 
 void
-gui_headers_set (GtkWidget         * table,
-		 const Article     * article,
-		 gboolean            need_lock,
-		 gulong              header_fields)
+gui_headers_set_nolock (GtkWidget         * table,
+		        const Article     * article,
+		        gulong              header_fields)
 {
 	const gchar * newsgroups;
 	const gchar * pch;
@@ -177,25 +173,20 @@ gui_headers_set (GtkWidget         * table,
 
 	g_return_if_fail (table != NULL);
 
-	if (need_lock)
-		pan_lock();
-
 	/* remove children */
 	gtk_container_foreach (
 		GTK_CONTAINER (table),
 		GTK_SIGNAL_FUNC (gtk_widget_destroy), NULL);
 
 	/* if no article, that's all we need to do */
-	if (article == NULL) {
-		pan_unlock ();
+	if (article == NULL)
 		return;
-	}
 
 	/* if author turned on, show it unconditionally */
 	if (header_fields & UI_HEADER_AUTHOR)
 	{
 		gchar * author = article_get_author_str (article);
-		add_header_nolock (table, row++, _("From: "), author, '\0', max_header_width);
+		add_header_nolock (table, row++, _("From: "), author, ' ', max_header_width);
 		g_free (author);
 	}
 
@@ -226,9 +217,9 @@ gui_headers_set (GtkWidget         * table,
 	/* if date turned on, show it unconditionally. */
 	if (header_fields & UI_HEADER_DATE)
 	{
-		gchar * date = get_date_display_string (article->date, body_date_format);     
-		add_header_nolock (table, row++, _("Date: "), date, '\0', max_header_width);
-		g_free (date);
+		gchar buf[256];
+		get_date_display_string (article->date, body_date_format, buf, sizeof(buf));
+		add_header_nolock (table, row++, _("Date: "), buf, '\0', max_header_width);
 	}
 
 	/* if reply_to turned on, and reply_to exists and != author, show it */
@@ -254,16 +245,12 @@ gui_headers_set (GtkWidget         * table,
 		add_header_nolock (table, row++, _("Subject: "), article_get_subject(article), ' ', max_header_width);
 	}
 
-	if (need_lock)
-		pan_unlock();
-
 	debug_exit ("gui_headers_set");
 }
 
 void
-gui_headers_set_default (GtkWidget      * table,
-			 const Article  * article,
-			 gboolean         need_lock)
+gui_headers_set_default_nolock (GtkWidget      * table,
+			        const Article  * article)
 {
-	gui_headers_set (table, article, need_lock, header_flags);
+	gui_headers_set_nolock (table, article, header_flags);
 }
