@@ -25,6 +25,7 @@
 #include <gnome.h>
 
 #include <pan/base/debug.h>
+#include <pan/base/pan-i18n.h>
 #include <pan/base/pan-glib-extensions.h>
 
 #include <pan/articlelist.h>
@@ -36,24 +37,30 @@
 #include <pan/text.h>
 #include <pan/util.h>
 
-
-void
-gui_paned_select_all (void)
+static GtkWidget*
+pack_widgets (GtkWidget * w1, GtkWidget * w2, GtkOrientation orient)
 {
-	switch (gui_paned_get_current_pane ())
-	{
-		case 0: /* groups tab */
-			grouplist_select_all ();
-			break;
-		case 1: /* articles tab */
-			articlelist_select_all ();
-			break;
-		case 2: /* message tab */
-			text_select_all ();
-			break;
-		default:
-			break;
+	GtkWidget * w;
+	gboolean vertical = orient == GTK_ORIENTATION_VERTICAL;
+
+	if (w1!=NULL && w2!=NULL) {
+		gint pos;
+		w = vertical ? gtk_vpaned_new() : gtk_hpaned_new ();
+		gtk_paned_pack1 (GTK_PANED(w), w1, FALSE, TRUE);
+		gtk_paned_pack2 (GTK_PANED(w), w2, TRUE, FALSE);
+		pos = vertical
+			?  gnome_config_get_int ("/Pan/Geometry/vpaned=302")
+			:  gnome_config_get_int ("/Pan/Geometry/hpaned=265");
+		gtk_paned_set_position (GTK_PANED(w), pos);
+	} else if (w1!=NULL) {
+		w = w1;
+	} else if (w2!=NULL) {
+		w = w2;
+	} else {
+		w = NULL;
 	}
+
+	return w;
 }
 
 
@@ -64,6 +71,7 @@ void
 gui_paned_construct (void)
 {
 	GtkWidget * w [3];
+	GtkWidget * work;
 	const gchar * pch;
 	int i;
 	int vpos;
@@ -72,11 +80,6 @@ gui_paned_construct (void)
 
 	pan_lock ();
 
-	/* ref the three main widgets so that reparenting them won't kill them */
-	gtk_widget_ref (groups_vbox);
-	gtk_widget_ref (articlelist_ctree);
-	gtk_widget_ref (text_box);
-
 	/* clear out the workarea */
 	gtk_widget_unrealize (GTK_WIDGET(contents_vbox));
 	gtk_widget_hide_all (GTK_WIDGET(contents_vbox));
@@ -91,10 +94,6 @@ gui_paned_construct (void)
 	gtk_widget_unrealize (text_box);
 
 	/* make a new UI */
-#if 0
-	contents_vbox = gtk_vbox_new (FALSE, 0);
-	gnome_app_set_contents (GNOME_APP(Pan.window), contents_vbox);
-#else
 	if (1) {
 		GList * children = gtk_container_children (GTK_CONTAINER(contents_vbox));
 		GList * l;
@@ -102,7 +101,6 @@ gui_paned_construct (void)
 			gtk_container_remove (GTK_CONTAINER(contents_vbox), GTK_WIDGET(l->data));
 		g_list_free (l);
 	}
-#endif
 
 	gtk_widget_show (groups_vbox);
 	gtk_widget_show (articlelist_ctree);
@@ -120,11 +118,11 @@ gui_paned_construct (void)
 	pch = layout_str + 1;
 	for (i=0; i<=2; ++i, ++pch) {
 		if (tolower(*pch)=='g')
-			w[i] = groups_vbox;
+			w[i] = show_group_pane ? groups_vbox : NULL;
 		else if (tolower(*pch)=='t')
-			w[i] = articlelist_ctree;
+			w[i] = show_header_pane ? articlelist_ctree : NULL;
 		else
-			w[i] = text_box;
+			w[i] = show_article_pane ? text_box : NULL;
 	}
 
 	vpos = gnome_config_get_int_with_default ("/Pan/Geometry/vpaned=302", NULL);
@@ -135,82 +133,45 @@ gui_paned_construct (void)
 	{
 		case '1':
 		{
-			GtkWidget * v = gtk_vpaned_new ();
-			GtkWidget * h = gtk_hpaned_new ();
-			gtk_paned_pack1 (GTK_PANED(h), w[0], FALSE, TRUE);
-			gtk_paned_pack2 (GTK_PANED(h), w[1], TRUE, FALSE);
-			gtk_paned_set_position (GTK_PANED(h), hpos);
-			gtk_paned_pack1 (GTK_PANED(v), h, FALSE, FALSE);
-			gtk_paned_pack2 (GTK_PANED(v), w[2], TRUE, FALSE);
-			gtk_paned_set_position (GTK_PANED(v), vpos);
-			gtk_box_pack_start (GTK_BOX(contents_vbox), v, TRUE, TRUE, 0);
+			GtkWidget * h = pack_widgets (w[0], w[1], GTK_ORIENTATION_HORIZONTAL);
+			work = pack_widgets (h, w[2], GTK_ORIENTATION_VERTICAL);
 			break;
 		}
 		case '2':
 		{
-			GtkWidget * v = gtk_vpaned_new ();
-			GtkWidget * h = gtk_hpaned_new ();
-			gtk_paned_pack1 (GTK_PANED(h), w[1], FALSE, FALSE);
-			gtk_paned_pack2 (GTK_PANED(h), w[2], TRUE, FALSE);
-			gtk_paned_set_position (GTK_PANED(h), hpos);
-			gtk_paned_pack1 (GTK_PANED(v), w[0], FALSE, FALSE);
-			gtk_paned_pack2 (GTK_PANED(v), h, TRUE, FALSE);
-			gtk_paned_set_position (GTK_PANED(v), vpos);
-			gtk_box_pack_start (GTK_BOX(contents_vbox), v, TRUE, TRUE, 0);
+			GtkWidget * h = pack_widgets (w[1], w[2], GTK_ORIENTATION_HORIZONTAL);
+			work = pack_widgets (w[0], h, GTK_ORIENTATION_VERTICAL);
 			break;
 		}
 		case '3':
 		{
-			GtkWidget * v = gtk_vpaned_new ();
-			GtkWidget * h = gtk_hpaned_new ();
-			gtk_paned_pack1 (GTK_PANED(v), w[0], FALSE, FALSE);
-			gtk_paned_pack2 (GTK_PANED(v), w[1], TRUE, FALSE);
-			gtk_paned_set_position (GTK_PANED(v), vpos);
-			gtk_paned_pack1 (GTK_PANED(h), v, FALSE, TRUE);
-			gtk_paned_pack2 (GTK_PANED(h), w[2], TRUE, FALSE);
-			gtk_paned_set_position (GTK_PANED(h), hpos);
-			gtk_box_pack_start (GTK_BOX(contents_vbox), h, TRUE, TRUE, 0);
+			GtkWidget * v = pack_widgets (w[0], w[1], GTK_ORIENTATION_VERTICAL);
+			work = pack_widgets (v, w[2], GTK_ORIENTATION_HORIZONTAL);
 			break;
 		}
 		case '5':
 		{
-			GtkWidget * v1 = gtk_vpaned_new ();
-			GtkWidget * v2 = gtk_vpaned_new ();
-			gtk_paned_pack1 (GTK_PANED(v2), w[1], FALSE, FALSE);
-			gtk_paned_pack2 (GTK_PANED(v2), w[2], TRUE, FALSE);
-			gtk_paned_pack1 (GTK_PANED(v1), w[0], FALSE, FALSE);
-			gtk_paned_pack2 (GTK_PANED(v1), v2, TRUE, FALSE);
-			gtk_paned_set_position (GTK_PANED(v1), vpos);
-			gtk_paned_set_position (GTK_PANED(v2), hpos);
-			gtk_box_pack_start (GTK_BOX(contents_vbox), v1, TRUE, TRUE, 0);
+			GtkWidget * v = pack_widgets (w[0], w[1], GTK_ORIENTATION_VERTICAL);
+			work = pack_widgets (v, w[2], GTK_ORIENTATION_VERTICAL);
 			break;
 		}
 		case '4':
 		default:
 		{
-			GtkWidget * v = gtk_vpaned_new ();
-			GtkWidget * h = gtk_hpaned_new ();
-			gtk_paned_pack1 (GTK_PANED(v), w[1], FALSE, FALSE);
-			gtk_paned_pack2 (GTK_PANED(v), w[2], TRUE, FALSE);
-			gtk_paned_set_position (GTK_PANED(v), vpos);
-			gtk_paned_pack1 (GTK_PANED(h), w[0], FALSE, TRUE);
-			gtk_paned_pack2 (GTK_PANED(h), v, TRUE, FALSE);
-			gtk_paned_set_position (GTK_PANED(h), hpos);
-			gtk_box_pack_start (GTK_BOX(contents_vbox), h, TRUE, TRUE, 0);
+			GtkWidget * v = pack_widgets (w[1], w[2], GTK_ORIENTATION_VERTICAL);
+			work = pack_widgets (w[0], v, GTK_ORIENTATION_HORIZONTAL);
 			break;
 		}
 	}
 
+	if (work != NULL)
+		gtk_box_pack_start (GTK_BOX(contents_vbox), work, TRUE, TRUE, 0);
+
 	Pan.viewmode = GUI_PANED;
 	gui_restore_column_widths (Pan.group_clist, "group");
 	gui_restore_column_widths (Pan.article_ctree, "thread_pane");
 	gtk_widget_show_all (GTK_WIDGET(contents_vbox));
 
-	/* balance the previous ref */
-	gtk_widget_unref (groups_vbox);
-	gtk_widget_unref (articlelist_ctree);
-	gtk_widget_unref (text_box);
-
 	pan_unlock ();
 	debug_exit ("gui_paned_construct");
 }
@@ -239,7 +200,7 @@ gui_paned_get_current_pane (void)
  * ENTER key pressed while in the paned mode, process accordingly
  *--------------------------------------------------------------------*/
 void
-gui_paned_enter_clicked (void)
+gui_paned_enter_clicked_nolock (void)
 {
 	switch (gui_paned_get_current_pane())
 	{
@@ -247,14 +208,14 @@ gui_paned_enter_clicked (void)
 		{
 			Group * g = grouplist_get_selected_group ();
 			if (g != NULL)
-				articlelist_set_group_thread (g);
+				articlelist_set_group_nolock (g);
 			break;
 		}
 		case 1: /* article tree */
 		{
-			Article* a = articlelist_get_selected_article ();
+			Article* a = articlelist_get_selected_article_nolock ();
 			if (a != NULL)
-				text_set_from_article (a, NNTP_READ, FALSE);
+				text_set_from_article (a);
 			break;
 		}
 		case 2: /* message body */
