@@ -25,11 +25,11 @@
 #include <string.h>
 #include <sys/types.h>
 
-#include <langinfo.h>
-
 #include <glib.h>
 
+#include <gmime/gmime-charset.h>
 #include <gmime/gmime-utils.h>
+#include <gmime/internet-address.h>
 
 #include <pan/base/debug.h>
 #include <pan/base/gnksa.h>
@@ -79,11 +79,16 @@ build_nntp_header (GString        * postme,
 	debug_enter ("build_nntp_header");
 
 	/* From: */
-	tmp = article_get_author_str (article);
-	pan_warn_if_fail (is_nonempty_string (tmp));
-	replace_gstr (&tmp, g_mime_utils_8bit_header_encode(tmp));
-	g_string_sprintfa (postme, "From: %s\r\n", tmp);
-	g_free (tmp);
+	if (1) {
+		InternetAddress * ia = internet_address_new_name (
+				article->author_real,
+				article->author_addr);
+		tmp = internet_address_to_string (ia, TRUE);
+		pan_warn_if_fail (is_nonempty_string (tmp));
+		g_string_sprintfa (postme, "From: %s\r\n", tmp);
+		g_free (tmp);
+		internet_address_destroy (ia);
+	}
 
 	/* Subject: */
 	pch = article_get_subject (article);
@@ -152,12 +157,9 @@ build_nntp_header (GString        * postme,
 	g_string_append (postme, "User-Agent: Pan/" VERSION " (Unix)\r\n");
 
 	/* Add mime header, charset type, encoding type */
-	g_string_append (postme, "Mime-Version: 1.0\r\n");
-	pch = nl_langinfo (CODESET);
-	if (!is_nonempty_string (pch))
-		pch = "ISO-8859-1";
-	g_string_sprintfa (postme, "Content-Type: text/plain; charset=%s\r\n", pch);
-	g_string_append (postme, "Content-Transfer-Encoding: 8bit\r\n");
+	g_string_sprintfa (postme, "Mime-Version: 1.0\r\n");
+	g_string_sprintfa (postme, "Content-Type: text/plain; charset=%s\r\n", g_mime_charset_locale_name());
+	g_string_sprintfa (postme, "Content-Transfer-Encoding: 8bit\r\n");
 
 	/* Extra headers: */
 	if (1) {
@@ -202,7 +204,7 @@ build_nntp_message (Server * server, Article * article)
 	debug_enter ("build_nntp_message");
 
 	/* sanity checks... */
-	g_return_val_if_fail (article!=NULL, NULL);
+	g_return_val_if_fail (article_is_valid(article), NULL);
 	g_return_val_if_fail (article_get_header(article,HEADER_NEWSGROUPS), 0);
 
        	bodies = g_ptr_array_new();
@@ -516,7 +518,7 @@ queue_article_for_posting (Server     * server,
 
 	/* sanity check */
 	g_return_val_if_fail (server!=NULL, 1);
-	g_return_val_if_fail (article!=NULL, 1);
+	g_return_val_if_fail (article_is_valid(article), 1);
 
 	/* if desired, make sure the article is okay. */
        	errors = g_ptr_array_new ();
@@ -575,10 +577,16 @@ flush_sendlater_articles (void)
 	for (i=0; i!=articles->len; ++i)
 	{
 		Article * a;
-		Server * s;
+		Server * s = NULL;
+		const gchar * h;
 	       
 		a = ARTICLE(g_ptr_array_index(articles,i));
-		s = serverlist_get_named_server (article_get_header (a, PAN_SERVER));
+		h = article_get_header (a, PAN_SERVER);
+
+		if (is_nonempty_string (h)) {
+			s = serverlist_get_named_server (h);
+		}
+			
 		if (s == NULL) {
 			g_message (_("Article \"%s\" not flagged for any particular server; using current"),
 				article_get_subject (a));
