@@ -21,6 +21,7 @@
 #include <config.h>
 
 #include <ctype.h>
+#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
@@ -177,7 +178,6 @@ nntp_disconnect (StatusItem   * status,
 	debug_enter ("nntp_disconnect");
 
 	/* entry assertions */
-	g_return_val_if_fail (status!=NULL, TASK_FAIL_HOPELESS);
 	g_return_val_if_fail (socket!=NULL, TASK_FAIL_HOPELESS);
 
 	/* try to disconnect */
@@ -381,7 +381,7 @@ nntp_post (StatusItem    * status,
 		/* tell the server we want to post */
 		val = nntp_command (status, sock, &response, "POST");
 		if (val != 340) {
-			status_item_emit_error_va (status, "Posting failed: %s", response);
+			status_item_emit_error_va (status, _("Posting failed: %s"), response);
 			return val==-1 ? TASK_FAIL : TASK_FAIL_HOPELESS;
 		}
 
@@ -396,6 +396,7 @@ nntp_post (StatusItem    * status,
 	}
 
 	/* if we made it this far, we succeeded! */
+	debug_exit ("nntp_post");
 	return TASK_SUCCESS;
 }
 
@@ -464,12 +465,13 @@ nntp_article_download (StatusItem      * status,
 	gint retval;
 	gint val;
 	GString * buf;
+	GString * line;
 	debug_enter ("nntp_article_download");
 
 	/* sanity clause */
 	g_return_val_if_fail (status!=NULL, TASK_FAIL);
 	g_return_val_if_fail (sock!=NULL, TASK_FAIL);
-	g_return_val_if_fail (group!=NULL, TASK_FAIL);
+	g_return_val_if_fail (group_is_valid(group), TASK_FAIL);
 	g_return_val_if_fail (article_is_valid(article), TASK_FAIL);
 	g_return_val_if_fail (abort!=NULL, TASK_FAIL);
 
@@ -496,6 +498,7 @@ nntp_article_download (StatusItem      * status,
 
 	/* try to read the article... */
 	getting_headers = TRUE;
+	line = g_string_sized_new (80);
 	buf = g_string_sized_new (80*400); /* prealloc space for ~400 lines */
 	retval = TASK_SUCCESS;
 	if (verbose & NNTP_VERBOSE_INIT_STEPS)
@@ -503,35 +506,40 @@ nntp_article_download (StatusItem      * status,
 
 	for (;;)
 	{
-		gchar * line;
-		gchar * freeme;
+		const gchar * pch;
 
 		/* check for end cases: user abort, socket error, and success */
 		if (*abort || pan_socket_getline (sock, &response)) {
 			retval = TASK_FAIL;
 			break;
 		}
+
+		if (!is_nonempty_string(response))
+			continue;
+
 		if (!strncmp(response, ".\r\n", 3)) {
 			retval = TASK_SUCCESS;
 			break;
 		}
 
 		/* strip out the \r */
-		freeme = line = pan_substitute (response, "\r\n", "\n");
+		g_string_assign (line, response);
+		while ((pch = strstr (line->str, "\r\n")))
+			g_string_erase (line, pch-line->str, 1);
 
 		/* rfc 977: 2.4.1 */
-		if (line[0]=='.' && line[1]=='.')
-			++line;
+		if (line->len>=2 && line->str[0]=='.' && line->str[1]=='.')
+			g_string_erase (line, 0, 2);
 
 		/* save the line */
-		g_string_append (buf, line);
+		g_string_append (buf, line->str);
 		if (verbose & NNTP_VERBOSE_NEXT_STEP)
 			status_item_emit_next_step (status);
 
 		/* save the headers that we want to save */
 		if (getting_headers)
 		{
-			if (!strcmp(line, "\n")) /* header/body separator */
+			if (line->len==1 && !strcmp(line->str, "\n")) /* header/body separator */
 				getting_headers = FALSE;
 			else 
 			{
@@ -539,33 +547,32 @@ nntp_article_download (StatusItem      * status,
 				gboolean matched = FALSE;
 
 				/* trim linefeed */
-				g_strstrip (line);
+				g_strstrip (line->str);
+				line->len = strlen(line->str);
 
 				/* check for headers to store in Article */
 				if (!matched)
-					matched = init_header_if_match (a, line,
+					matched = init_header_if_match (a, line->str,
 					                                "Newsgroups: ",
 					                                HEADER_NEWSGROUPS,
 					                                DO_CHUNK_SHARE);
 				if (!matched)
-					matched = init_header_if_match (a, line,
+					matched = init_header_if_match (a, line->str,
 					                                "Reply-To: ",
 					                                HEADER_REPLY_TO,
 					                                DO_CHUNK_SHARE);
 				if (!matched)
-					matched = init_header_if_match (a, line,
+					matched = init_header_if_match (a, line->str,
 					                                "Followup-To: ",
 					                                HEADER_FOLLOWUP_TO,
 					                                DO_CHUNK_SHARE);
 				if (!matched)
-					matched = init_header_if_match (a, line,
+					matched = init_header_if_match (a, line->str,
 					                                "Xref: ",
 					                                HEADER_XREF,
 					                                DO_CHUNK);
 			}
 		}
-
-		g_free (freeme);
 	}
 
 	/* save the message */
@@ -575,6 +582,7 @@ nntp_article_download (StatusItem      * status,
 	}
 
 	g_string_free (buf, TRUE);
+	g_string_free (line, TRUE);
 	debug_exit ("nntp_article_download");
 	return retval;
 }
@@ -646,7 +654,7 @@ nntp_download_headers (StatusItem       * status,
 	/* sanity checks */
 	g_return_val_if_fail (status!=NULL, TASK_FAIL_HOPELESS);
 	g_return_val_if_fail (sock!=NULL, TASK_FAIL);
-	g_return_val_if_fail (group!=NULL, TASK_FAIL_HOPELESS);
+	g_return_val_if_fail (group_is_valid(group), TASK_FAIL_HOPELESS);
 	g_return_val_if_fail (abort!=NULL, TASK_FAIL_HOPELESS);
 	g_return_val_if_fail (is_nonempty_string(progress_fmt), TASK_FAIL_HOPELESS);
 	g_return_val_if_fail (setme!=NULL, TASK_FAIL_HOPELESS);
@@ -698,76 +706,87 @@ nntp_download_headers (StatusItem       * status,
 	        gint parts = 0;
 		const gchar * pch;
 		const gchar * s;
-		gchar * tmp;
 		Article * a;
+		gchar * buf = NULL;
+		gchar * dec = NULL;
+		const gchar * march = NULL;
+		const gchar * tok = NULL;
+		int tok_len = 0;
 
 		/* handle end-of-list */
 		if (!strncmp(buffer, ".\r\n", 3))
 			break;
 
 		/* create the article data */
-		pch = buffer;
 		a = article_new (group);
 		group_set_articles_dirty (group);
 
-		/* article number */
-		a->number = get_next_token_ulong (pch, '\t', &pch);
+		/* setup for parsing */
+		buf = g_strdup (buffer);
+		march = buf;
 
-		/* subject */
-		tmp = get_next_token_str (pch, '\t', &pch);
-		replace_gstr (&tmp, g_mime_utils_8bit_header_decode(tmp));
-		if (!is_nonempty_string(tmp))
-			replace_gstr (&tmp, g_strdup(_("No Subject")));
-		article_init_header (a, HEADER_SUBJECT, tmp, DO_CHUNK_SHARE);
-		g_free (tmp);
+		/* get article number */
+		a->number = get_next_token_ulong (march, '\t', &march);
 
-		/* author */
-		tmp = get_next_token_str (pch, '\t', &pch);
-		if (is_nonempty_string(tmp))
-			article_init_author_from_header (a, tmp);
-		g_free (tmp);
-
-		/* date */
-		tmp = get_next_token_str (pch, '\t', &pch);
-		a->date = is_nonempty_string(tmp)
-			? g_mime_utils_header_decode_date (tmp, NULL)
-			: (time_t)0;
-		g_free (tmp);
-
-		/* message id */
-		tmp = get_next_token_str (pch, '\t', &pch);
-		if (is_nonempty_string(tmp) && *tmp=='<')
-			article_init_header (a, HEADER_MESSAGE_ID, tmp, DO_CHUNK);
-		g_free (tmp);
-
-		/* references */
-		tmp = get_next_token_str (pch, '\t', &pch);
-		if (is_nonempty_string(tmp) && *tmp=='<')
-			article_init_header (a, HEADER_REFERENCES, tmp, DO_CHUNK_SHARE);
-		g_free (tmp);
-
-		/* unused field: size */
-		skip_next_token (pch, '\t', &pch);
-
-		/* linecount */
-		a->linecount = get_next_token_int(pch, '\t', &pch);
-
-		/* crosspost */
-		tmp = get_next_token_str (pch, '\t', &pch);
-		if (tmp==NULL || strncmp (tmp, "Xref: ", 6))
-			a->crosspost_qty = 1;
-		else
-		{
-			const gchar * foo;
-
-			a->crosspost_qty = 0;
-			for (foo=tmp+6; *foo; ++foo)
-				if (*foo==':')
-					++a->crosspost_qty;
-
-			article_init_header (a, HEADER_XREF, tmp+6, DO_CHUNK);
+		/* get subject */
+		if (get_next_token_range (march, '\t', &march, &tok, &tok_len)) {
+			((gchar*)tok)[tok_len] = '\0';
+			dec = g_mime_utils_8bit_header_decode (tok);
+			article_init_header (a, HEADER_SUBJECT, dec, DO_CHUNK_SHARE);
+			replace_gstr (&dec, NULL);
+		}
+
+		/* get author */
+		if (get_next_token_range (march, '\t', &march, &tok, &tok_len)) {
+			((gchar*)tok)[tok_len] = '\0';
+			if (is_nonempty_string(tok))
+				article_init_author_from_header (a, tok);
+		}
+
+		/* get date */
+		if (get_next_token_range (march, '\t', &march, &tok, &tok_len)) {
+			((gchar*)tok)[tok_len] = '\0';
+			a->date = is_nonempty_string(tok)
+				? g_mime_utils_header_decode_date (tok, NULL)
+				: (time_t)0;
+		}
+
+		/* get message id */
+		if (get_next_token_range (march, '\t', &march, &tok, &tok_len)) {
+			((gchar*)tok)[tok_len] = '\0';
+			if (is_nonempty_string(tok) && *tok=='<')
+				article_init_header (a, HEADER_MESSAGE_ID, tok, DO_CHUNK);
+		}
+
+		/* get references */
+		if (get_next_token_range (march, '\t', &march, &tok, &tok_len)) {
+			if (tok_len>0 ) {
+				((gchar*)tok)[tok_len] = '\0';
+				if (is_nonempty_string(tok) && *tok=='<')
+					article_init_header (a, HEADER_REFERENCES, tok, DO_CHUNK_SHARE);
+			}
+		}
+
+		/* get unused field: size */
+		skip_next_token (march, '\t', &march);
+
+		/* get linecount */
+		a->linecount = get_next_token_int (march, '\t', &march);
+
+		/* get crossref */
+		if (get_next_token_range (march, '\t', &march, &tok, &tok_len)) {
+			((gchar*)tok)[tok_len] = '\0';
+			if (tok_len!=0 || strncmp(tok,"Xref: ", 6))
+				a->crosspost_qty = 1;
+			else {
+				const gchar * foo;
+				a->crosspost_qty = 0;
+				for (foo=tok+6; *foo; ++foo)
+					if (*foo==':')
+						++a->crosspost_qty;
+				article_init_header (a, HEADER_XREF, tok+6, DO_CHUNK);
+			}
 		}
-		g_free (tmp);
 
 		/**
 		***  Validity Checking
@@ -775,8 +794,7 @@ nntp_download_headers (StatusItem       * status,
 
 		if (!article_is_valid (a)) {
 			status_item_emit_error_va (status, 
-				_("Corrupt header skipped - no Message Id: %s"), buffer);
-			pan_object_unref (PAN_OBJECT(a));
+				_("Corrupt header skipped: %s"), buffer);
 			continue;
 		}
 
@@ -849,8 +867,11 @@ nntp_download_headers (StatusItem       * status,
 
 		/* update the count & progress ui */
 		status_item_emit_next_step (status);
-		if (!(setme->len % 29))
+		if (!(setme->len % 57))
 			status_item_emit_status_va (status, progress_fmt, setme->len, (high-low));
+
+		/* cleanup */
+		g_free (buf);
 	}
 
 	if (read_status!=0)
@@ -884,7 +905,7 @@ nntp_download_bodies (StatusItem       * status,
 
 	/* sanity checks */
 	g_return_val_if_fail (status!=NULL, TASK_FAIL_HOPELESS);
-	g_return_val_if_fail (group!=NULL, TASK_FAIL_HOPELESS);
+	g_return_val_if_fail (group_is_valid(group), TASK_FAIL_HOPELESS);
 	g_return_val_if_fail (sock!=NULL, TASK_FAIL);
 	g_return_val_if_fail (abort!=NULL, TASK_FAIL_HOPELESS);
 	g_return_val_if_fail (articles!=NULL, TASK_FAIL_HOPELESS);
@@ -894,11 +915,8 @@ nntp_download_bodies (StatusItem       * status,
 	if (*abort)
 		return TASK_FAIL;
 	result = nntp_set_group (status, sock, group->name);
-	if (result != TASK_SUCCESS) {
-		const gchar * g = group_get_readable_name (group);
-		status_item_emit_error_va (status, _("Couldn't select group \"%s\""), g);
+	if (result != TASK_SUCCESS)
 		return result;
-	}
 
 	/* status item */
 	lines_so_far = lines_total = 0;
