@@ -27,13 +27,13 @@
 #include <pan/base/newsrc-port.h>
 #include <pan/base/serverlist.h>
 #include <pan/base/util-file.h>
-
 #include <pan/filters/filter-manager.h>
-
 #include <pan/rules/rule-manager.h>
-
+#include <pan/identities/identity-manager.h>
 #include <pan/dialogs/dialogs.h>
 
+#include <gmime/gmime-charset.h>
+
 #include <pan/globals.h>
 #include <pan/gui.h>
 #include <pan/pan.h>
@@ -42,10 +42,6 @@
 #include <pan/thread-watch.h>
 #include <pan/util.h>
 
-#ifdef GTKHTML_HAVE_GCONF
-#include <gconf/gconf-client.h>
-#endif
-
 static gint save_session (GnomeClient        * client,
 			  gint                 phase,
 			  GnomeSaveStyle       save_style,
@@ -138,14 +134,12 @@ main (int argc, char *argv[])
 	g_thread_init (NULL);
 	gnome_init_with_popt_table ("Pan", VERSION, argc, argv, options, 0, &pctx);
 	poptFreeContext (pctx);
+	g_mime_charset_init ();
 
 #if 0 /* for rpm */
 #if GNOME_LIBS_VERSION>=60
 	gnome_window_icon_set_default_from_file (GNOME_ICONDIR "/pan.png");
 #endif
-#endif
-#ifdef GTKHTML_HAVE_GCONF
-	gconf_init (argc, argv, NULL);
 #endif
 
 	debug_flags = (debug_acache ? DEBUG_ACACHE : 0)
@@ -164,13 +158,11 @@ main (int argc, char *argv[])
 
 	log_add_va (LOG_INFO, _("Pan %s Started"), VERSION);
 
+	cmap = gdk_colormap_get_system ();
 	prefs_init ();
 	filter_manager_init ();
 	serverlist_init ();
 
-
-	check_and_warn_if_version_change ();
-
 	/* Connect up to the session manager */
 	client = gnome_master_client ();
 	gtk_signal_connect (GTK_OBJECT (client), "save_yourself",
@@ -190,7 +182,8 @@ main (int argc, char *argv[])
 
 	/* if it's a new user, pop up the newuser dialog.
 	   Otherwise show the main window. */
-	if (!gnome_config_has_section ("/Pan/User"))
+
+	if (identity_manager_count () == 0)
 	{
 		dialog_newuser ();
 	}
@@ -237,6 +230,7 @@ pan_shutdown (void)
 	g_free (thread_new_replies_font);
 	g_free (grouplist_font);
 
+	identity_manager_shutdown_module ();
 	serverlist_shutdown ();
 	acache_shutdown ();
 	thread_shutdown_module ();
