@@ -393,7 +393,7 @@ apply_now_thread (void * data)
 		{
 			status_item_emit_status_va (status, _("Refreshing group `%s'"),
 				group_get_readable_name(g));
-			articlelist_reload();
+			articlelist_refresh_nolock ();
 		}
 
 		status_item_emit_next_step (status);
