@@ -37,7 +37,6 @@
 #include <sys/signal.h>
 #include <sys/types.h>
 #include <fcntl.h>
-#include <pthread.h>
 
 #include <pan/base/debug.h>
 #include <pan/base/log.h>
@@ -52,52 +51,16 @@
 
 #define BUFSIZE 1024
 
-/*********************
-**********************  Macros
-*********************/
-
-/*********************
-**********************  Structures / Typedefs
-*********************/
-
-/*********************
-**********************  Private Function Prototypes
-*********************/
-
 /*********************
 **********************  Variables
 *********************/
 
-/***********
-************  Extern
-***********/
-
-/***********
-************  Public
-***********/
-
-/***********
-************  Private
-***********/
-
 static gulong total_bytes = 0;
 
-#if 0
-static int reads_til_fail = 500;
-#endif
-
 /*********************
 **********************  BEGINNING OF SOURCE
 *********************/
 
-/************
-*************  PUBLIC ROUTINES
-************/
-
-/*****
-******
-*****/
-
 PanSocket*
 pan_socket_new (const gchar    * address,
                 gint             port)
@@ -326,7 +289,16 @@ pan_socket_getline (PanSocket     * sock,
 	register char * pch;
 	register gboolean done = FALSE;
 
+	/* sanity clause */
+	g_return_val_if_fail (sock!=NULL, -1);
+	g_return_val_if_fail (setme!=NULL, -1);
+
+	/* zero out the buffer */
+	if (sock->line_buffer->data != NULL)
+		*sock->line_buffer->data = '\0';
 	g_array_set_size (sock->line_buffer, 0);
+
+	/* set the action clock */
 	sock->last_action_time = time(0);
 
 	while (!done)
@@ -436,6 +408,24 @@ pan_socket_putline (PanSocket     * sock,
 	return nleft;
 }
 
+gint
+fd_write (gint fd, const gchar * buf, gint len)
+{
+	gint n, wrlen = 0;
+		 
+	while (len) {
+		n = write(fd, buf, len);
+		if (n <= 0)
+			return -1;
+		len -= n;
+		wrlen += n;
+		buf += n;
+	}
+
+	return wrlen;
+}
+
+
 gboolean 
 pan_socket_putline_va (PanSocket    *sock,
                        const gchar  *format,
