@@ -29,8 +29,11 @@
 #include <string.h>
 #include <stdlib.h>
 
-#include <gnome.h>
+#include <gtk/gtkeventbox.h>
+#include <gtk/gtkprogress.h>
+#include <gtk/gtkprogressbar.h>
 
+#include <pan/base/argset.h>
 #include <pan/base/debug.h>
 
 #include <pan/gui.h>
@@ -62,8 +65,9 @@ static gchar* string_shorten_nolock (GdkFont* font, int width, const gchar* orig
 
 static void status_item_view_unregister (StatusItemView *view);
 
-static int progress_cb      (gpointer, gpointer, gpointer);
-static int status_cb        (gpointer, gpointer, gpointer);
+static void progress_cb      (gpointer, gpointer, gpointer);
+static void status_cb        (gpointer, gpointer, gpointer);
+static void status_cb_nolock (gpointer, gpointer, gpointer);
 
 /*********************
 **********************  Variables
@@ -89,6 +93,58 @@ static GtkObjectClass *parent_class = NULL;
 **********************  BEGINNING OF SOURCE
 *********************/
 
+/************
+*************  PRIVATE ROUTINES
+************/
+
+/**
+ * Structure holding the arguments to gtk_proress_set_percentage.
+ */
+typedef struct
+{
+	GtkProgress * progress;
+	gfloat f;
+}
+IdleStruct;
+
+/**
+ * Call gtk_progress_set_percentage in an idle func.
+ */
+static gint
+set_percentage_mainthread (gpointer p)
+{
+	IdleStruct * data = (IdleStruct*) p;
+
+	pan_lock_unconditional ();
+	if (GTK_IS_PROGRESS(data->progress))
+	{
+		gtk_progress_set_percentage (data->progress, data->f);
+		gtk_widget_unref (GTK_WIDGET(data->progress));
+	}
+	pan_unlock_unconditional ();
+
+	g_free (data);
+	return 0;
+}
+
+/**
+ * Defer the call to an idle func to ensure we're in the right thread.
+ */
+static void
+set_percentage_nolock (GtkProgress * progress, gfloat f)
+{
+	IdleStruct * s;
+
+	g_return_if_fail (GTK_IS_PROGRESS(progress));
+
+	s = g_new0 (IdleStruct, 1);
+	s->progress = progress;
+	s->f = f;
+
+	gtk_widget_ref (GTK_WIDGET(progress));
+	run_in_main_thread_nolock (set_percentage_mainthread, s);
+}
+
 /************
 *************  PUBLIC ROUTINES
 ************/
@@ -140,6 +196,7 @@ status_item_view_init (StatusItemView *view)
 	view->progress = NULL;
 	view->last_status = NULL;
 	view->item = NULL;
+	view->of_100 = -1;
 }
 
 
@@ -224,8 +281,10 @@ status_item_view_unregister (StatusItemView *view)
 }
 
 void
-status_item_view_set_item (StatusItemView* view, StatusItem* item)
+status_item_view_set_item_nolock (StatusItemView* view, StatusItem* item)
 {
+	debug_enter ("status_item_view_set_item_nolock");
+
 	if (item != view->item)
 	{
 		GtkProgress* prog = GTK_PROGRESS(view->progress);
@@ -233,7 +292,7 @@ status_item_view_set_item (StatusItemView* view, StatusItem* item)
 		status_item_view_unregister (view);
 
 		/* update last_status; update progressbar w/o a resize */
-		gtk_progress_set_percentage (prog, 0);
+		set_percentage_nolock (prog, 0);
 		replace_gstr (&view->last_status, g_strdup(""));
 		replace_gstr (&prog->format, g_strdup(""));
 		GTK_PROGRESS_CLASS (GTK_OBJECT(prog)->klass)->update (prog);
@@ -251,65 +310,72 @@ status_item_view_set_item (StatusItemView* view, StatusItem* item)
 		}
 
 		view->item = item;
+
+		/* show the last status text */
+		if (item !=NULL)
+		{
+			gchar * pch = g_strdup (item->status_text);
+			if (is_nonempty_string(pch))
+				status_cb_nolock (NULL, pch, view);
+			g_free (pch);
+		}
 	}
+
+	debug_exit ("status_item_view_set_item_nolock");
 }
 
 /*****
 ******
 *****/
 
-static gint
-progress_cb (gpointer call_object,
-             gpointer call_arg,
-             gpointer user_data)
+static void
+progress_cb (gpointer call_object, gpointer call_arg, gpointer user_data)
 {
 	const gint of_100 = GPOINTER_TO_INT(call_arg);
-
-	/* update the percentage */
 	if (0<=of_100 && of_100<=100)
 	{
-		StatusItemView * view;
-		GtkProgress * prog;
-
-		pan_lock();
- 		view = STATUS_ITEM_VIEW(user_data);
-		prog = GTK_PROGRESS(view->progress);
-		gtk_progress_set_percentage (prog, of_100/100.0);
-		pan_unlock();
+		StatusItemView * siv = STATUS_ITEM_VIEW(user_data);
+		pan_lock ();
+		set_percentage_nolock (GTK_PROGRESS(siv->progress), of_100/100.0);
+		pan_unlock ();
 	}
+}
+
+static gint
+status_idle (gpointer data)
+{
+	StatusItemView * view;
+
+	pan_lock_unconditional ();
+	view = STATUS_ITEM_VIEW(data);
+	if (view!=NULL && view->item!=NULL)
+	{
+		GtkProgress * prog = GTK_PROGRESS(view->progress);
+		gchar * s = string_shorten_nolock (gtk_widget_get_style (view->progress)->font,
+		                                   view->progress->allocation.width,
+		                                   view->last_status);
+		replace_gstr (&prog->format, s);
+		/* update progressbar w/o a resize */
+		GTK_PROGRESS_CLASS (GTK_OBJECT(prog)->klass)->update (prog);
+	}
+	pan_unlock_unconditional ();
 
 	return 0;
 }
 
-/**
-***
-**/
-
-static int
-status_cb (gpointer call_object,
-           gpointer call_arg,
-           gpointer user_data)
+static void
+status_cb_nolock (gpointer call_object, gpointer call_arg, gpointer user_data)
+{
+	StatusItemView * siv = STATUS_ITEM_VIEW (user_data);
+	replace_gstr (&siv->last_status, g_strdup ((const gchar*)call_arg));
+	run_in_main_thread_nolock (status_idle, siv);
+}
+static void
+status_cb (gpointer call_object, gpointer call_arg, gpointer user_data)
 {
 	pan_lock ();
-	if (1)
-	{
-		StatusItemView * v = STATUS_ITEM_VIEW(user_data);
-		GtkProgress * prog = GTK_PROGRESS(v->progress);
-		const gchar * str = (const gchar*)call_arg;
-		char * s = string_shorten_nolock (
-			gtk_widget_get_style(v->progress)->font,
-			v->progress->allocation.width,
-			str);
-
-		/* update last_status; update progressbar w/o a resize */
-		replace_gstr (&v->last_status, g_strdup(str));
-		replace_gstr (&prog->format, s);
-		GTK_PROGRESS_CLASS (GTK_OBJECT(prog)->klass)->update (prog);
-	}
-	pan_unlock();
-
-	/* cleanup */
-	return 0;
+	status_cb_nolock (call_object, call_arg, user_data);
+	pan_unlock ();
 }
 
 /**
@@ -338,7 +404,7 @@ string_shorten_nolock (GdkFont           * font,
 	size_t len = 0;
 	gint i, j;
 	gboolean i_turn = TRUE;
-	gint target_width = max_width - 4 - 20; /* 20 is just a magic safety number */
+	gint target_width = max_width - 4 - 30; /* 30 is just a magic safety number */
 
 	/* if original is NULL, so is shortened */
 	if (!original)
