@@ -36,24 +36,11 @@
 #include <pan/base/pan-i18n.h>
 #include <pan/base/pan-glib-extensions.h>
 
-#include <pan/message-window.h>
 #include <pan/nntp.h>
 #include <pan/queue.h>
 #include <pan/task-body.h>
 #include <pan/text.h>
 
-/*********************
-**********************  Defines / Enumerated types
-*********************/
-
-/*********************
-**********************  Macros
-*********************/
-
-/*********************
-**********************  Structures / Typedefs
-*********************/
-
 /*********************
 **********************  Private Function Prototypes
 *********************/
@@ -62,22 +49,6 @@ static int task_body_run (Task * task);
 
 static gchar * task_body_describe (const StatusItem * status);
 
-/*********************
-**********************  Variables
-*********************/
-
-/***********
-************  Extern
-***********/
-
-/***********
-************  Public
-***********/
-
-/***********
-************  Private
-***********/
-
 /*********************
 **********************  BEGINNING OF SOURCE
 *********************/
@@ -96,7 +67,6 @@ task_body_destructor (PanObject * o)
 
 	/* unref the group + article */
 	group = task->article->group;
-	pan_object_unref (PAN_OBJECT(task->article));
 	task->article = NULL;
 	group_unref_articles (group, STATUS_ITEM(task));
 
@@ -108,16 +78,9 @@ task_body_destructor (PanObject * o)
 *************  PUBLIC ROUTINES
 ************/
 
-PanObject*
-task_body_new_defaults (Article  * article)
-{
-	return task_body_new (article, FALSE, NNTP_READ);
-}
 
 PanObject*
-task_body_new (Article            * a,
-               gboolean             open_in_new_window,
-               MessageWindowType    message_window_type)
+task_body_new (Article * a)
 {
 	TaskBody * body = NULL;
 	gboolean body_exists;
@@ -130,12 +93,11 @@ task_body_new (Article            * a,
 	message_id = article_get_message_id (a);
 
 	/* sanity clause */
-	g_return_val_if_fail (a!=NULL, NULL);
-	g_return_val_if_fail (a->group!=NULL, NULL);
+	g_return_val_if_fail (article_is_valid(a), NULL);
+	g_return_val_if_fail (group_is_valid(a->group), NULL);
 	g_return_val_if_fail (is_nonempty_string(message_id), NULL);
 
 	/* ref the group + article */
-	pan_object_ref (PAN_OBJECT(a));
 	group_ref_articles (a->group, NULL);
 
 	/* do we already have the body for this message? */
@@ -164,8 +126,6 @@ task_body_new (Article            * a,
 	/* initialize the task-body */
 	debug1 (DEBUG_PAN_OBJECT, "task_body: %p", a);
 	body->article = a;
-	body->open_in_new_window = open_in_new_window;
-	body->message_window_type = message_window_type;
 
 	debug_exit ("task_body_new");
 	return PAN_OBJECT(body);
@@ -187,7 +147,7 @@ task_body_describe (const StatusItem * status)
 	/* sanity clause */
 	g_return_val_if_fail (status!=NULL, NULL);
 	g_return_val_if_fail (article_is_valid(TASK_BODY(status)->article), NULL);
-	g_return_val_if_fail (TASK_BODY(status)->article->group!=NULL, NULL);
+	g_return_val_if_fail (group_is_valid(TASK_BODY(status)->article->group), NULL);
 
 	message_id = article_get_message_id (TASK_BODY(status)->article);
 	subject = article_get_subject (TASK_BODY(status)->article);
@@ -241,9 +201,9 @@ task_body_run (Task * task)
 	
 	/* sanity clause */
 	g_return_val_if_fail (server!=NULL, TASK_FAIL_HOPELESS);
-	g_return_val_if_fail (group!=NULL, TASK_FAIL_HOPELESS);
+	g_return_val_if_fail (group_is_valid(group), TASK_FAIL_HOPELESS);
 	g_return_val_if_fail (is_nonempty_string(group->name), TASK_FAIL_HOPELESS);
-	g_return_val_if_fail (article!=NULL, TASK_FAIL_HOPELESS);
+	g_return_val_if_fail (article_is_valid(article), TASK_FAIL_HOPELESS);
 	g_return_val_if_fail (is_nonempty_string(message_id), TASK_FAIL_HOPELESS);
 
 	/* let the user know what's happening */
@@ -297,8 +257,7 @@ task_body_run (Task * task)
 			{
 				GString * tmp = g_string_sized_new (1024);
 				GSList * l = status->errors;
-				while (l != NULL)
-				{
+				while (l != NULL) {
 					g_string_append (tmp, (gchar*)l->data);
 					g_string_append_c (tmp, '\n');
 					l = l->next;
@@ -312,17 +271,8 @@ task_body_run (Task * task)
 		}
 	}
 
-	/**
-	***  Display the message?
-	**/
-
-	if (body->open_in_new_window)
-		 /* message window */
-		message_window_new (article, body->message_window_type);
-	else
-		/* use default text window */
-		text_update (article);
-
+	/* read the message */
+	text_set_from_cached_article (article);
 	articles_set_read (&article, 1, TRUE);
 
 	debug_exit ("task_body_run");
