@@ -1,7 +1,7 @@
 #ifndef THREAD_ARTICLE_H
 #define THREAD_ARTICLE_H
 
-#include "article.h"
+#include <pan/base/article.h>
 
 enum {
 	THREAD_NOTHING,
