@@ -74,10 +74,6 @@ void pan_unlock_unconditional_from (const gchar * file, int line);
 
 
 
-gchar* get_default_author_address (void);
-
-gchar* get_default_author_from (void);
-
 void pan_info_dialog (const char * fmt, ...);
 
 void pan_error_dialog (const char * fmt, ...);
@@ -85,19 +81,13 @@ void pan_error_dialog (const char * fmt, ...);
 void pan_error_dialog_parented (gpointer gtkwindow, const char * fmt, ...);
 
 
-char* get_fqdn (const char * host);
-
-char* get_host_name (void);
-
 void open_outside_file (const gchar * filename);
 
 gchar* linkify_text (const gchar * text);
 
 void replace_gstr (char ** target_free_old, char * assign_from_me);
 
-void check_and_warn_if_version_change (void);
-
-gchar* get_date_display_string (time_t date, const gchar * strftime_fmt);
+void get_date_display_string (time_t date, const gchar * strftime_fmt, gchar * buf, gint size);
 
 void pan_url_show (const gchar * url);
 
